/* ------------------------------------------------------------------        *
 *                                                                           * 
 *   Copyright (C) 1995 Roger Lindell                                        * 
 *   All rights reserved.                                                    * 
 *                                                                           * 
 *   Adapted by Roger Lindell at the Dept. of Speech, Music and Hearing,     * 
 *   KTH (Royal Institute of Technology), Sweden, from an original algorithm *
 *   described in "Programs for Digital Signal Processing", Bergland & Dolan *
 *   (1979) Ed. by the DSP committee IEEE Acoustics, Speech and Signal       *
 *   Processing Society. Wiley, New York.                                    *
 *   Roger Lindell, rog@speech.kth.se                                        * 
 *                                                                           * 
 *   KTH                                                                     * 
 *   Institutionen foer Tal, musik och hoersel                               * 
 *   Box 700 14                                                              * 
 *   100 44 STOCKHOLM                                                        * 
 *   SWEDEN                                                                  * 
 *                                                                           * 
 *---------------------------------------------------------------------------*
 * Function Snack_DBPowerSpectrum                                            *
 *    Fast Fourier Transform for N=2**M                                      *
 *    Complex Input                                                          *
 *---------------------------------------------------------------------------*
 *                                                                           *
 *   This program replaces the vector z=x+iy by its  finite                  *
 *   discrete, complex fourier transform if in=0.  The inverse               *
 *   transform is calculated for in=1.  It performs as many base             *
 *   8 iterations as possible and then finishes with a base                  *
 *   4 iteration or a base 2 iteration if needed.                            *
 *                                                                           *
 *---------------------------------------------------------------------------*
 *                                                                           */

/*                                                                           *
 *---------------------------------------------------------------------------*
 *  Modified by zhuo in 2020                                                 *
 *---------------------------------------------------------------------------*
 * API                                                                       *
 *   The functions are called as                                             *
 *        Snack_InitFFT( n ) ;     / * int n must be a power of 2 * /        *
 *   Then, either                                                            *
 *        float x[n];                                                        *
 *        Snack_PowerSpectrum ( x ) ;                                        *
 *   or                                                                      *
 *        float x[n];                                                        *
 *        Snack_DBPowerSpectrum ( x ) ;                                      *
 *                                                                           *
 *   Actually,                                                               *
 *        Snack_PowerSpectrum ( x ) ;                                        *
 *   is identical to                                                         *
 *        Snack_ComplexSpectrum ( x ) ;  / * result is stored in x  * /      *
 *        Snack_ComplexToPower ( x ) ;                                       *
 *   and                                                                     *
 *        Snack_DBPowerSpectrum ( x ) ;                                      *
 *   is identical to                                                         *
 *        Snack_ComplexSpectrum ( x ) ;  / * result is stored in x  * /      *
 *        Snack_ComplexToPower ( x ) ;   / * result is stored in x * /       *
 *        Snack_fLinToDB ( x, -1 ) ;     / * result is stored in x * /       *
 *        By giving -1, the length set initially for x is used internally.   *
 *                                                                           *
 *   Also there are following utility functions:                             *
 *     void Snack_fLinToDB( float* x, int len ) ;                            *
 *       // converts each element in x from linear to DB                     *
 *     void Snack_dLinToDB( double* x, int len ) ;                           *
 *       // double version of fLinToDB.                                      *
 *     void Snack_fDBToLin( float* x, int len ) ;                            *
 *       // converts each element in x from DB to linear                     *
 *     void Snack_dDBToLin( double* x, int len ) ;                           *
 *       // double version of fDBToLin.                                      *
 *     void Snack_setLogScale( float* boundaries, int len,                   *
 *                             float bottom_idx, float top_idx ) ;           *
 *       // fills boundaries[0] .. [len] (len + 1 elements) with             *
 *       // logarithmic series of  bottom_idx, ...., top_idx.                *
 *     void Snack_rescale( float *in, int inlen,                             *
 *                             float *out, float *boundaries, int len ) ;    *
 *       // fills out[n] with the average value of in[] between              *
 *       // boundaries[n] and [n+1], where n goes from 0 to len-1.           *
 *       // Average is given as area / width, where area is the integral     *
 *       // of in[], STEP-interpolated.                                      *
 *     void Snack_rescaleInterp( float *in, int inlen,                       *
 *                             float *out, float *boundaries, int len ) ;    *
 *       // the same as Snack_rescale(), but in[] is LINEAR-interpolated.    *
 *                                                                           *
 *---------------------------------------------------------------------------*
 *NOTES:                                                                     *
 *   1) By defining __FFA_NO_SNACK__ upon compilation, this module becomes   *
 *     independent on any other software packages like snack or tcl so it    *
 *     can be used as a versatile fft module.                                *
 *   2) Orignial interfaces,  PowerSpectrum() and DBPowerSpectrum(),         *
 *     remain unchanged in functionality but now these are implemented       *
 *     using ComplexSpectrum(), ComplexToPower(), and fLinToDB(z, nthpo).    *
 *   3) In addition, the following functions are introduced:                 *
 *     void Snack_fLinToDB( float* z, len ) ;   linear to dB conversion      *
 *     void Snack_dLinToDB( double* z, len ) ;  linear to dB conversion      *
 *     void Snack_fDBToLin( float* z, len ) ;   dB to linear conversion      *
 *     void Snack_dDBToLin( double* z, len ) ;  dB to linear conversion      *
 *     void setLogScale( ... );   set boundary table for log conversion      *
 *     void rescale( ... )  fill array with new boundaries (0th order)       *
 *     void rescaleInterp( ... ) fill array with new boundaries (1st order)  *
 *                                                                           *
 *   * rescale() and rescaleInterp() are coarse approximations of CQT        *
 *     (Constant Q Transform). CQT should be implemented for a better        *
 *     result in the future.  See, for instance:                             *
 *     Judith C Brown and Miller Puckette, "An efficient algorithm for the   *
 *     calculation of a constant Q transform", November 1992,                *
 *     The Journal of the Acoustical Society of America 92(5):2698           *
 *     https://www.researchgate.net/publication/230554907_An_efficient_algorithm_for_the_calculation_of_a_constant_Q_transform
 *                                                                           *
 *---------------------------------------------------------------------------*
 *                                                                           */
#ifdef __cplusplus
extern "C" {
#endif
  int  Snack_InitFFT(int n);
  void Snack_DBPowerSpectrum(float* x);
  void Snack_PowerSpectrum(float* x);
  void Snack_ComplexSpectrum(float* x);
  void Snack_ComplexToPower( float* z );
  void Snack_fLinToDB( float* x, int len );
  void Snack_dLinToDB( double* x, int len );
  void Snack_fDBToLin( float* x, int len );
  void Snack_dDBToLin( double* x, int len );
  void Snack_setLogScale( float* boundaries, int len, float bottom_idx, float top_idx );
  void Snack_rescale( float *in, int inlen, float *out, float *boundaries, int len );
  void Snack_rescaleInterp( float *in, int inlen, float *out, float *boundaries, int len );
#ifdef __cplusplus
}
#endif
